@@ -1,5 +1,5 @@
 # -*- Mode: cperl; cperl-indent-level: 4 -*-
-# $Id: Harness.pm,v 1.13 2001/07/31 03:01:46 schwern Exp $
+# $Id: Harness.pm,v 1.17 2001/09/07 06:20:29 schwern Exp $
 
 package Test::Harness;
 
@@ -20,7 +20,7 @@ use vars qw($VERSION $Verbose $Switches $Have_Devel_Corestack $Curtest
 
 $Have_Devel_Corestack = 0;
 
-$VERSION = "1.23";
+$VERSION = 1.25;
 
 $ENV{HARNESS_ACTIVE} = 1;
 
@@ -289,7 +289,7 @@ sub runtests {
     my($tot, $failedtests) = _run_all_tests(@tests);
     _show_results($tot, $failedtests);
 
-    my $ok = ($tot->{bad} == 0 && $tot->{max});
+    my $ok = _all_ok($tot);
 
     die q{Assert '$ok xor keys %$failedtests' failed!}
       unless $ok xor keys %$failedtests;
@@ -299,6 +299,20 @@ sub runtests {
 
 =begin _private
 
+=item B<_all_ok>
+
+  my $ok = _all_ok(\%tot);
+
+Tells you if this test run is overall successful or not.
+
+=cut
+
+sub _all_ok {
+    my($tot) = shift;
+
+    return $tot->{bad} == 0 && ($tot->{max} || $tot->{skipped}) ? 1 : 0;
+}
+
 =item B<_globdir>
 
   my @files = _globdir $dir;
@@ -328,6 +342,7 @@ and values are this:
     max             Number of individual tests ran
     ok              Number of individual tests passed
     sub_skipped     Number of individual tests skipped
+    todo            Number of individual todo tests
 
     files           Number of test files ran
     good            Number of test files passed
@@ -371,6 +386,7 @@ sub _run_all_tests {
                 good     => 0,
                 tests    => scalar @tests,
                 sub_skipped  => 0,
+                todo     => 0,
                 skipped  => 0,
                 bench    => 0,
                );
@@ -395,8 +411,20 @@ sub _run_all_tests {
     my @dir_files = _globdir $Files_In_Dir if defined $Files_In_Dir;
     my $t_start = new Benchmark;
 
+    my $maxlen = 0;
+    my $maxsuflen = 0;
+    foreach (@tests) { # The same code in t/TEST
+       my $suf    = /\.(\w+)$/ ? $1 : '';
+       my $len    = length;
+       my $suflen = length $suf;
+       $maxlen    = $len    if $len    > $maxlen;
+       $maxsuflen = $suflen if $suflen > $maxsuflen;
+    }
+    # + 3 : we want three dots between the test name and the "ok"
+    my $width = $maxlen + 3 - $maxsuflen;
+
     foreach my $tfile (@tests) {
-        my($leader, $ml) = _mk_leader($tfile);
+        my($leader, $ml) = _mk_leader($tfile, $width);
         print $leader;
 
         my $fh = _open_test($tfile);
@@ -531,22 +559,25 @@ sub _run_all_tests {
 
 =item B<_mk_leader>
 
-  my($leader, $ml) = _mk_leader($test_file);
+  my($leader, $ml) = _mk_leader($test_file, $width);
 
 Generates the 't/foo........' $leader for the given $test_file as well
 as a similar version which will overwrite the current line (by use of
 \r and such).  $ml may be empty if Test::Harness doesn't think you're
 on TTY.
 
+The $width is the width of the "yada/blah.." string.
+
 =cut
 
 sub _mk_leader {
-    my $te = shift;
-    chop($te);      # XXX chomp?
+    my($te, $width) = @_;
+    chomp($te);
+    $te =~ s/\.\w+$/./;
 
     if ($^O eq 'VMS') { $te =~ s/^.*\.t\./\[.t./s; }
     my $blank = (' ' x 77);
-    my $leader = "$te" . '.' x (20 - length($te));
+    my $leader = "$te" . '.' x ($width - length($te));
     my $ml = "";
 
     $ml = "\r$blank\r$leader"
@@ -562,7 +593,7 @@ sub _show_results {
     my $pct;
     my $bonusmsg = _bonusmsg($tot);
 
-    if ($tot->{bad} == 0 && $tot->{max}) {
+    if (_all_ok($tot)) {
         print "All tests successful$bonusmsg.\n";
     } elsif (!$tot->{tests}){
         die "FAILED--no tests were run for some reason.\n";
@@ -572,9 +603,10 @@ sub _show_results {
             "alas--no output ever seen\n";
     } else {
         $pct = sprintf("%.2f", $tot->{good} / $tot->{tests} * 100);
+        my $percent_ok = 100*$tot->{ok}/$tot->{max};
         my $subpct = sprintf " %d/%d subtests failed, %.2f%% okay.",
                               $tot->{max} - $tot->{ok}, $tot->{max}, 
-                              100*$tot->{ok}/$tot->{max};
+                              $percent_ok;
 
         my($fmt_top, $fmt) = _create_fmts($failedtests);
 
@@ -668,7 +700,7 @@ sub _parse_test_line {
     my($line, $test, $tot) = @_;
 
     if ($line =~ /^(not\s+)?ok\b/i) {
-        $test->{next} ||= 1;
+        $test->{'next'} ||= 1;
         my $this = $test->{'next'};
         # "not ok 23"
         if ($line =~ /^(not )?ok\s*(\d*)[^#]*(\s*#.*)?/) {
@@ -681,11 +713,13 @@ sub _parse_test_line {
 
             my($istodo, $isskip);
             if( defined $type ) {
-                $istodo = $type =~ /TODO/;
-                $isskip = $type =~ /skip/i;
+                $istodo = 1 if $type =~ /TODO/;
+                $isskip = 1 if $type =~ /skip/i;
             }
 
-            $test->{todo}{$tnum} = 1 if $istodo;
+            $test->{todo}{$this} = 1 if $istodo;
+
+            $tot->{todo}++ if $test->{todo}{$this};
 
             if( $not ) {
                 print "$test->{ml}NOK $this" if $test->{ml};
